"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
const path = require("path");
const chalk = require("chalk");
const inquirer = require("inquirer");
const cli_utils_1 = require("@ionic/cli-utils");
const commands_1 = require("../commands");
const globalPlugin = require("../index");
exports.defaultPlugin = 'core';
exports.KNOWN_PLUGINS = [exports.defaultPlugin, 'cordova'];
exports.PREFIX = '@ionic/cli-plugin-';
exports.ERROR_PLUGIN_NOT_INSTALLED = 'PLUGIN_NOT_INSTALLED';
exports.ERROR_PLUGIN_NOT_FOUND = 'PLUGIN_NOT_FOUND';
exports.ERROR_PLUGIN_INVALID = 'PLUGIN_INVALID';
/**
 * Synchronously load a plugin
 */
function loadPlugin(projectDir, name, askToInstall = true) {
    return __awaiter(this, void 0, void 0, function* () {
        let m;
        try {
            var mPath = require.resolve(path.join(projectDir, 'node_modules', '@ionic', `cli-plugin-${name}`));
            m = require(mPath);
        }
        catch (e) {
            if (e.code !== 'MODULE_NOT_FOUND') {
                throw e;
            }
            // Unfortuantly we need to check the not only the code but the error message
            let foundPackageNeeded = exports.KNOWN_PLUGINS.map(kp => `${exports.PREFIX}${kp}`)
                .find(kp => e.message && e.message.includes(kp));
            if (!foundPackageNeeded) {
                throw `Dependency missing for ${chalk.bold(exports.PREFIX + name)}:\n\n  ${chalk.red('[ERROR]')}: ${e.message}`;
            }
        }
        if (!m && !askToInstall) {
            throw exports.ERROR_PLUGIN_NOT_INSTALLED;
        }
        if (!m) {
            const pluginName = `${exports.PREFIX}${name}`;
            const answers = yield inquirer.prompt([{
                    type: 'confirm',
                    name: 'installPlugin',
                    message: `This command's plugin ${chalk.green(pluginName)} is not installed would you like to install it and continue?`
                }]);
            if (answers['installPlugin']) {
                const releaseChannelName = yield getReleaseChannelName();
                const pluginInstallVersion = `${pluginName}` + (releaseChannelName ? `@${releaseChannelName}` : '');
                const tasks = new cli_utils_1.TaskChain();
                tasks.next(`Executing npm command: ${chalk.bold(`npm install --save-dev ${pluginInstallVersion}`)}`);
                yield new cli_utils_1.Shell().run('npm', ['install', '--save-dev', pluginInstallVersion]);
                tasks.end();
                return loadPlugin(projectDir, name);
            }
            else {
                throw exports.ERROR_PLUGIN_NOT_INSTALLED;
            }
        }
        return m;
    });
}
exports.loadPlugin = loadPlugin;
/**
 * Get inputs and command class based on arguments supplied
 */
function resolvePlugin(projectDir, argv) {
    return __awaiter(this, void 0, void 0, function* () {
        let pluginName;
        let inputs = [];
        /**
         * If this module's primary namespace has the command then use it.
         */
        const ionicNamespace = new commands_1.IonicNamespace();
        const isGlobalCmd = ionicNamespace.getCommands().has(argv[0]);
        if (isGlobalCmd || argv.length === 0) {
            return [
                globalPlugin,
                argv
            ];
        }
        // If this we do not know the project directory and it is not a global command then we can't run it.
        if (!projectDir) {
            throw chalk.bold(`This is not a global command please run this in your Ionic project's directory.\n`);
        }
        /**
         * If the first arguement supplied contains a ':' then
         * it is assumed that this is calling a command in another
         * namespace. <namespaceName>:<commandName>
         *
         * Else it is likely a core command that should be loaded
         * from the 'core' plugin.
         */
        if (argv[0].includes(':')) {
            let [firstArg, ...restOfArguments] = argv;
            [pluginName, firstArg] = firstArg.split(':');
            inputs = [firstArg, ...restOfArguments];
        }
        else if (exports.KNOWN_PLUGINS.includes(argv[0])) {
            [pluginName, ...inputs] = argv;
        }
        else {
            pluginName = exports.defaultPlugin;
            inputs = argv;
        }
        /**
         * Load the plugin using the pluginName provided
         */
        try {
            return [
                yield loadPlugin(projectDir, pluginName),
                inputs
            ];
        }
        catch (e) {
            /**
             * If plugin is not found then lets make a recommendation
             * based on whether we know the plugin exists.
             */
            if (e === exports.ERROR_PLUGIN_NOT_INSTALLED) {
                const releaseChannelName = yield getReleaseChannelName();
                const pluginInstallVersion = `${exports.PREFIX}${pluginName}` + (releaseChannelName ? `@${releaseChannelName}` : '');
                throw new cli_utils_1.FatalException('This plugin is not currently installed. Please execute the following to install it.\n\n'
                    + `    ${chalk.bold(`npm install --save-dev ${pluginInstallVersion}`)}\n`);
            }
            else if (e === exports.ERROR_PLUGIN_NOT_FOUND) {
                throw new cli_utils_1.FatalException(`Unknown plugin: ${chalk.bold(exports.PREFIX + pluginName)}.`);
            }
            throw e;
        }
    });
}
exports.resolvePlugin = resolvePlugin;
function getReleaseChannelName() {
    return __awaiter(this, void 0, void 0, function* () {
        let jsonStructure;
        const filePath = path.resolve(__dirname, '..', '..', 'package.json');
        try {
            jsonStructure = yield cli_utils_1.fsReadJsonFile(filePath);
        }
        catch (e) {
            if (e === cli_utils_1.ERROR_FILE_NOT_FOUND) {
                throw new Error(`${filePath} not found`);
            }
            else if (e === cli_utils_1.ERROR_FILE_INVALID_JSON) {
                throw new Error(`${filePath} is not valid JSON.`);
            }
            throw e;
        }
        if (jsonStructure.version.includes('alpha')) {
            return 'canary';
        }
        if (jsonStructure.version.includes('beta')) {
            return 'beta';
        }
    });
}
exports.getReleaseChannelName = getReleaseChannelName;
